#!/usr/bin/env bash

set -e

unset LD_{RUN,LIBRARY}_PATH LIBRARY_PATH CPATH {C,CPLUS}_INCLUDE_PATH PYTHONPATH
unset "${!EUPS_@}" "${!LSST_@}" SETUP_EUPS

pushd "$(dirname -- "$0")"
  scriptdir="$PWD"
popd

source "$scriptdir/lib/install.rc"
source "$scriptdir/libversions"
source "$scriptdir/variables"

if [[ $nproc -ge 2 ]]
then
  export MAKEFLAGS="-j $(($nproc))"
  export EUPSPKG_NJOBS="$(($nproc))"
  # export EUPSPKG_SCONSFLAGS="-j $(($nproc)) opt=3"
fi

workdir="${workdir:-/tmp}"/hscpipe_build_dir

lsst_home="${prefix}/lsst_home"


main()
(
  export PATH="$prefix/bin:$path"

  install_gcc
  install_binutils
  install_git
  install_sqlite3
  install_python

  if [[ "$minimal_python_lib" = "False" ]]
  then
    # These libraries are not essential.
    # They are not necessary during the build process,
    # nor will users use them in a usual scenario.
    pip3 install \
      psycopg2-binary \
      ;
  fi

  # We need to pre-install several python libraries.
  # To know what to pre-install:
  #     'find {stack_dir}/Linux64 -name check_python_module'
  # and https://github.com/HyperSuprime-Cam/scipipe_conda_env/blob/master/etc/conda3_packages-linux-64.yml
  pip3 install \
    numpy \
    scipy \
    pandas \
    matplotlib \
    astropy \
    requests \
    pyarrow \
    ;
  # Some libraries must be of specific versions,
  # which must be installed last of all
  # because it is highly possible that they would be replaced by newer versions
  # on further calls to pip3.
  pip3 install \
    setuptools==47.3.1 \
    pytest==4.5.0 \
    pytest-arraydiff==0.3 \
    pytest-astropy==0.5.0 \
    pytest-cov==2.7.1 \
    pytest-doctestplus==0.3.0 \
    pytest-forked==1.0.2 \
    pytest-openfiles==0.3.2 \
    pytest-remotedata==0.3.1 \
    pytest-xdist==1.29.0 \
    deprecated==1.2.11 \
    urllib3==1.26.16 \
    ;
  export EUPS_PKGROOT_BASE_URL="${eups_pkgroot_base_url}"

  # install source patches
  find "$scriptdir/patches" -maxdepth 1 -name "*.patch" -print0 | while read -d '' patch
  do
    package="$(basename "$patch")"
    package="${package%.*}"
    if [[ ! -e "$workdir/eupsrepo/products/$package.eupspkg" ]]
    then
      bash "$scriptdir/clone_repo.sh" "$package"
      pushd "$workdir/eupsrepo/products"
        tar -xzf "$package.eupspkg"
        patch -p0 < "$patch"
        tar -czf - "$package" > "$package.eupspkg"
      popd
    fi
  done

  mkdir -p "${lsst_home}"
  pushd "${lsst_home}"

  install_eups

  # We replace the build directory (EupsBuildDir)
  # with a symbolic link to a subdirectory of $workdir
  if :
  then
    miniconda_dir=(stack/miniconda*)
    rm -rf "$miniconda_dir/EupsBuildDir"
    mkdir -p "$workdir/EupsBuildDir/Linux64"
    mkdir "$miniconda_dir/EupsBuildDir"
    ln -s "$workdir/EupsBuildDir/Linux64" "$miniconda_dir/EupsBuildDir"
  fi

  popd

  cleaner_thread "${lsst_home}/${miniconda_dir}" &

  source "${lsst_home}"/loadLSST.bash
  (
    set -e
    if ! env EUPS_PKGROOT="$workdir/eupsrepo|${EUPS_PKGROOT_BASE_URL}src" eups distrib install hscPipe "${hscpipe_ver}"
    then
      # If installation fails, we try it again with flake8 disabled.
      # flake8 (coding style checker) is run during builds,
      # which is useful in development, often reports errors
      # due to subtleties of library versions.
      # We disable it because coding style won't affect runtime behavior.
      status=false
      for flake8 in $(find "${lsst_home}" -type f -name "flake8-*.egg")
      do
        if [ -e "$(dirname "${flake8}")/flake8" ]
        then
          continue
        fi
        status=true
        pushd "$(dirname "${flake8}")"
        unzip "${flake8}"
        patch -p0 << __eof__
--- flake8/plugins/manager.py	2019-02-05 01:33:16.000000000 +0900
+++ flake8/plugins/manager.py	2021-03-15 10:32:36.343878586 +0900
@@ -453,9 +453,7 @@

         Find all checker plugins that are expecting a specific argument.
         """
-        for plugin in self.plugins.values():
-            if argument_name == plugin.parameter_names[0]:
-                yield plugin
+        return []

     def to_dictionary(self):
         """Return a dictionary of AST and line-based plugins."""
__eof__
        popd
      done
      $status
      echo "We disable flake8 and try to continue..."
      env EUPS_PKGROOT="$workdir/eupsrepo|${EUPS_PKGROOT_BASE_URL}src" eups distrib install hscPipe "${hscpipe_ver}"
    fi
  )

  rm -rf "${lsst_home}/${miniconda_dir}/EupsBuildDir"

  strip_hack_from_gcc

  # enable flake8 if it has been disabled
  for flake8 in $(find "${lsst_home}" -type f -name "flake8-*.egg")
  do
    if ! [ -e "$(dirname "${flake8}")/flake8" ]
    then
      continue
    fi
    pushd "$(dirname "${flake8}")"
    rm -rf flake8 EGG-INFO
    popd
  done

  # install bashrc
  cat "$(dirname "$0")"/rsrc/bashrc.in |
    sed s$'\xff'@prefix@$'\xff'"$prefix"$'\xff'g |
    sed s$'\xff'@lsst_home@$'\xff'"$LSST_HOME"$'\xff'g |
    sed s$'\xff'@hscpipe_ver@$'\xff'"$hscpipe_ver"$'\xff'g \
    > "$prefix"/bashrc

  # setup-eups() in bashrc needs $prefix/bin/check-requisites-hscPipe-${hscpipe_ver},
  # which will be created by post_build.sh. For users who won't run
  # post_build.sh, we place an empty exe there

  echo "int main(){ return 0; }" > "$workdir"/check-requisites.c
  gcc -O3 "$workdir"/check-requisites.c \
    -o "${prefix}/bin/check-requisites-hscPipe-${hscpipe_ver}"

  # Place create-rootdir.sh in bin/
  cat "$(dirname "$0")"/rsrc/create-rootdir.sh > "$prefix"/bin/create-rootdir.sh
  chmod a+x "$prefix"/bin/create-rootdir.sh

  # remove ".tests" directories
  find "${prefix}" -type d -name ".tests" -exec rm -rf {} +

  # remove build.log
  find "${prefix}" -path "*/ups/build.log" -delete
)

cleaner_thread()
(
  cd "$1"
  while [[ -e EupsBuildDir ]]
  do
    sleep 60
    dirs=($(ls -d -t -1 EupsBuildDir/Linux64/* 2> /dev/null))
    # remove all the dirs except the newest ones
    for((i = 3; i < ${#dirs[@]}; ++i))
    do
      rm -rf "${dirs[i]}"
    done
  done

  echo "EupsBuildDir cleaner died. (This is not an error)"
)

main
